<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:fo="http://www.w3.org/1999/XSL/Format" 
    xmlns:xep="http://www.renderx.com/XEP/xep"
    exclude-result-prefixes="xep" 
    version="1.0">

    <xsl:template name="scale-image-to-fit">
        <!-- the xep document with just images -->
        <xsl:param name="xepdoc"/>
        <!-- the scale-to width in pts -->
        <xsl:param name="fit-width"/>
        <!-- the scale-to height in pts -->
        <xsl:param name="fit-height"/>
        <!-- the name of the image to lookup in the XEP Intermediate File -->
        <xsl:param name="image-name"/>
        <fo:external-graphic>
            <!-- XEP Intermediate Format Conversion: 1 XEP Unit = 1/1000 pt -->
            <xsl:variable name="image-width"
                select="($xepdoc//xep:image[contains(@src, $image-name)]/@width) div 1000"/>
            <xsl:variable name="image-height"
                select="($xepdoc//xep:image[contains(@src, $image-name)]/@height) div 1000"/>
            <xsl:choose>
                <!-- Image fits in both dimensions -->
                <xsl:when
                    test="$image-width &lt; $fit-width and $image-height &lt; $fit-height">
                    <xsl:attribute name="width">
                        <xsl:value-of select="concat($image-width,'pt')"/>
                    </xsl:attribute>
                    <xsl:attribute name="height">
                        <xsl:value-of select="concat($image-height,'pt')"/>
                    </xsl:attribute>
                </xsl:when>
                <!-- Image is wider than fit area but not taller --> 
                <xsl:when test="$image-width > $fit-width and $image-height &lt; $fit-height">
                    <xsl:call-template name="scale-width">
                        <xsl:with-param name="fit-width" select="$fit-width"/>
                    </xsl:call-template>
                </xsl:when>
                <!-- Image is taller than fit area but not wider --> 
                <xsl:when test="$image-width &lt; $fit-width and $image-height > $fit-height">
                    <xsl:call-template name="scale-height">
                        <xsl:with-param name="fit-height" select="$fit-height"/>
                    </xsl:call-template>
                </xsl:when>
                <xsl:otherwise>
                    <!-- Image doesn't fit in both dimensions, figure out which scale will be best -->
                    <xsl:choose>
                        <xsl:when
                            test="($image-width div $fit-width) > ($image-height div $fit-height)">
                            <xsl:call-template name="scale-width">
                                <xsl:with-param name="fit-width" select="$fit-width"/>
                            </xsl:call-template>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:call-template name="scale-height">
                                <xsl:with-param name="fit-height" select="$fit-height"/>
                            </xsl:call-template>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:otherwise>
            </xsl:choose>
            <!-- Output the source element for the image -->
            <xsl:attribute name="src">
                <xsl:text>url('</xsl:text>
                <xsl:value-of select="$image-name"/>
                <xsl:text>')</xsl:text>
            </xsl:attribute>
        </fo:external-graphic>
    </xsl:template>

    <xsl:template name="scale-width">
        <xsl:param name="fit-width"/>
        <xsl:attribute name="width">
            <xsl:value-of select="concat($fit-width,'pt')"/>
        </xsl:attribute>
        <xsl:attribute name="content-height">100%</xsl:attribute>
        <xsl:attribute name="content-width">scale-to-fit</xsl:attribute>
        <xsl:attribute name="scaling">uniform</xsl:attribute>
    </xsl:template>

    <xsl:template name="scale-height">
        <xsl:param name="fit-height"/>
        <xsl:attribute name="height">
            <xsl:value-of select="concat($fit-height,'pt')"/>
        </xsl:attribute>
        <xsl:attribute name="content-width">100%</xsl:attribute>
        <xsl:attribute name="content-height">scale-to-fit</xsl:attribute>
        <xsl:attribute name="scaling">uniform</xsl:attribute>
    </xsl:template>

</xsl:stylesheet>
